import click
import subprocess
from pathlib import Path
import os


@click.group()
@click.pass_context
def query(ctx):
    """Node query commands. Will query the local node"""


@query.command()
@click.option('-r', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.pass_context
def tip(ctx, dry_run):
    """Get the node's current tip (slot no, hash, block no)."""

    cardano_node_socket_path = os.getenv('CARDANO_NODE_SOCKET_PATH')
    if not cardano_node_socket_path:
        ctx.fail(f"Invalid CARDANO_NODE_SOCKET_PATH. Undefined.")
        return 1

    cardano_node_socket_path = Path(cardano_node_socket_path)
    if not cardano_node_socket_path.exists():
        ctx.fail(f"Invalid CARDANO_NODE_SOCKET_PATH. '{cardano_node_socket_path}' does not exist.")
        return 1

    cardano_network = os.getenv('CARDANO_NETWORK')
    if not cardano_network:
        cardano_network = "testnet"

    cardano_network_cmd = ["--mainnet"] if cardano_network == "mainnet" else ["--testnet-magic", "1097911063"]

    query_tip_cmd = ["cardano-cli", "query", "tip"] + cardano_network_cmd
    if dry_run:
        print(" ".join(query_tip_cmd))
    else:
        try:
            subprocess.run(query_tip_cmd)
        except Exception as ex:
            ctx.fail(f"Unknown exception: {type(ex).__name__} {ex.args}")
            return 1




import click

from .run import run
from .exec import exec

@click.group()
@click.pass_context
def docker(ctx):
    """Docker helper command"""
    pass

# export
docker.add_command(run)
docker.add_command(exec)


import yaml
from pathlib import Path


class CardanoPyConfig(object):
    def __init__(self):
        pass

    target_config_yaml = None

    def load(self, target_config_file: Path):

        if not target_config_file.is_file():
            print(f"Target config '{target_config_file}' is not a file. e.g., 'cardanopy.yaml'")
            return False

        if not target_config_file.exists():
            print(f"Target config '{target_config_file}' does not exist.")
            return False

        with open(target_config_file, "r") as file:
            self.target_config_yaml = yaml.full_load(file.read())
            return True

        return False

    def set(self, property: str, value):
        self.target_config_yaml[property] = value

    def save(self, target_config_file: Path):

        if not target_config_file.is_file():
            print(f"Target config '{target_config_file}' is not a file. e.g., 'cardanopy.yaml'")
            return False

        with open(target_config_file, "w") as file:
            yaml.dump(self.target_config_yaml, file)
            return True

        return False

    def get_substitutions(self):
        return self.target_config_yaml['substitutions']

    substitutions = property(get_substitutions)

    def get_name(self):
        return self.target_config_yaml['name']

    name = property(get_name)

    def get_root(self):
        return self.target_config_yaml['root']

    root = property(get_root)

    def get_network(self):
        return self.target_config_yaml['network']

    network = property(get_network)

    def get_config(self):
        return self.target_config_yaml['config']

    config = property(get_config)

    def get_topologyPath(self):
        return self.target_config_yaml['topologyPath']

    topologyPath = property(get_topologyPath)

    def get_topology(self):
        return self.target_config_yaml['topology']

    topology = property(get_topology)

    def get_databasePath(self):
        return self.target_config_yaml['databasePath']

    databasePath = property(get_databasePath)

    def get_hostAddr(self):
        return self.target_config_yaml['hostAddr']

    hostAddr = property(get_hostAddr)

    def get_port(self):
        return self.target_config_yaml['port']

    port = property(get_port)

    def get_socketPath(self):
        return self.target_config_yaml['socketPath']

    socketPath = property(get_socketPath)

    def get_docker(self):
        return DockerConfig(self.target_config_yaml['docker'])

    docker = property(get_docker)


class DockerConfig(object):
    config = None

    def __init__(self, config):
        self.config = config

    def get_image(self):
        return self.config['image']

    image = property(get_image)



import click
from pathlib import Path
import shutil

_TEMPLATES_DIR = Path(__file__).parent.joinpath("templates").absolute()


@click.command()
@click.option('-t', '--template', 'template', required=True, type=str, help='template type to create.')
@click.option('-t', '--network', 'network', default="testnet",
              type=click.Choice(['testnet', 'mainnet'], case_sensitive=False), help='network type to create.')
@click.option('-r', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.argument('out_dir', type=click.Path(file_okay=False, dir_okay=True, exists=False))
@click.pass_context
def create(ctx, template, network, dry_run, out_dir):
    """Create command"""
    out_dir = Path(out_dir)
    network = network.lower()
    network_dir = _TEMPLATES_DIR.joinpath(network)

    if dry_run:
        print("#### DRY RUN - no mutable changes will be made. ####")

    if out_dir.exists():
        ctx.fail(f"Failed to create. Directory '{out_dir}' already exists.")
        return 1

    if network_dir.is_dir() and network_dir.exists():
        config_dir = network_dir.joinpath("config")
        storage_dir = network_dir.joinpath("storage")
        template_yaml = network_dir.joinpath(f"{template}.yaml")

        if not config_dir.is_dir() or not config_dir.exists():
            ctx.fail(f"Failed to create. Unable to locate template '{network}/config'.")
            return 1

        if not storage_dir.is_dir() or not storage_dir.exists():
            ctx.fail(f"Failed to create. Unable to locate template '{network}/storage'.")
            return 1

        if not template_yaml.is_file() or not template_yaml.exists():
            ctx.fail(f"Failed to create. Unable to locate template '{network}/{template}.yaml'.")
            return 1

        try:
            output_config_dir = out_dir.joinpath("config")
            if dry_run:
                print(f"copy 'config' directory from '{config_dir}' to '{output_config_dir}'")
            else:
                shutil.copytree(config_dir, output_config_dir)
        except Exception as ex:
            ctx.fail(f"Failed to create. Unable to copy config to '{out_dir}'. {type(ex).__name__} {ex.args}")
            return 1

        try:
            output_storage_dir = out_dir.joinpath("storage")
            if dry_run:
                print(f"copy 'storage' directory from '{storage_dir}' to '{output_storage_dir}'")
            else:
                shutil.copytree(storage_dir, output_storage_dir)
        except Exception as ex:
            ctx.fail(f"Failed to create. Unable to copy storage to '{out_dir}'. {type(ex).__name__} {ex.args}")
            return 1

        try:
            output_template_yaml = out_dir.joinpath('cardanopy.yaml')
            if dry_run:
                print(f"copy 'cardanopy.yaml' file from '{template_yaml}' to '{output_template_yaml}'")
            else:
                shutil.copyfile(template_yaml, output_template_yaml)
        except Exception as ex:
            ctx.fail(f"Failed to create. Unable to locate '{template}.yaml' to '{out_dir}'. {type(ex).__name__} {ex.args}")
            return 1

        if not dry_run:
            print(f"Created cardano defaults from '{template}' template for network '{network}': '{out_dir}'")
    else:
        ctx.fail(f"Failed to create. Unable to locate template '{template}' for network '{network}'.")
        return 1



import click

from .query import query
from .address import address
from .stake_address import stake_address

@click.group()
@click.pass_context
def cli(ctx):
    """Cardano CLI command"""
    pass

# export
cli.add_command(query)
cli.add_command(address)
cli.add_command(stake_address)


import click
import subprocess
import os

CARDANO_NETWORK = os.getenv('CARDANO_NETWORK')

@click.group()
@click.pass_context
def address(ctx):
    """Payment address commands"""

@address.command("key-gen")
@click.option('--normal-key', 'key_type', flag_value='normal-key', default=True, help='Use a normal Shelley-era key (default).')
@click.option('--extended-key', 'key_type', flag_value='extended-key', help='Use an extended ed25519 Shelley-era key.')
@click.option('--byron-key', 'key_type', flag_value='byron-key', help='Use a Byron-era key.')
@click.option('--verification-key-file', 'verification_key_file', required=True, type=str, help='Output filepath of the verification key.')
@click.option('--signing-key-file', 'signing_key_file', required=True, type=str, help='Output filepath of the signing key.')
def key_gen(key_type, verification_key_file, signing_key_file):
    """Create an address key pair."""
    subprocess.run(["cardano-cli",
                    "address",
                    "key-gen",
                    f"--{key_type}",
                    "--verification-key-file", verification_key_file,
                    "--signing-key-file", signing_key_file
                    ])


@address.command()
@click.pass_context
@click.option('--payment-verification-key-file', 'payment_verification_key_file', required=True, type=str, help='Output filepath of the verification key.')
@click.option('--stake-verification-key-file', 'stake_verification_key_file', required=True, type=str, help='Output filepath of the signing key.')
@click.option('--out-file', 'out_file', type=str, help='Optional output file. Default is to write to stdout')
def build(ctx, payment_verification_key_file, stake_verification_key_file, out_file):
    """Create an address key pair."""

    if CARDANO_NETWORK == "mainnet":
        cardano_network_params = ['--mainnet']
    elif CARDANO_NETWORK == "testnet":
        cardano_network_params = ['--testnet-magic', '1097911063']
    else:
        ctx.fail("missing or invalid param: mainnet or testnet")
        return

    subprocess.run(["cardano-cli",
                    "address",
                    "build",
                    "--payment-verification-key-file", payment_verification_key_file,
                    "--stake-verification-key-file", stake_verification_key_file]
                    + ([] if out_file is None else ['--out-file', out_file])
                    + cardano_network_params)


import click
from pathlib import Path
from .cardanopy_config import CardanoPyConfig


@click.command()
@click.option('-r', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('--config-filename', 'config_filename', default='cardanopy.yaml', type=str, help="defaults to 'cardanopy.yaml'")
@click.argument('property', type=str, required=True)
@click.argument('value', type=str, required=True)
@click.argument('target_dir', type=str)
@click.pass_context
def config(ctx, dry_run, property, value, target_dir, config_filename):
    """Config command"""
    if dry_run:
        print("#### DRY RUN - no mutable changes will be made. ####")

    target_dir = Path(target_dir)

    if not target_dir.is_dir():
        ctx.fail(f"Target directory '{target_dir}' is not a directory. e.g., the directory that contains 'cardanopy.yaml'")
        return 1

    if not target_dir.exists():
        ctx.fail(f"Target directory '{target_dir}' does not exist.")
        return 1

    target_config = target_dir.joinpath(config_filename)

    if not target_config.is_file():
        ctx.fail(
            f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")
        return 1

    if not target_config.exists():
        ctx.fail(f"Target file '{target_config}' does not exist.")
        return 1

    config = CardanoPyConfig()
    if not config.load(target_config):
        ctx.fail(f"Failed to load '{target_config}'")
        return 1

    config.set(property, value)

    if not config.save(target_config):
        ctx.fail(f"Failed to save '{target_config}'")
        return 1



import click
import subprocess

@click.group("stake-address")
@click.pass_context
def stake_address(ctx):
    """Stake address commands"""

@stake_address.command("key-gen")
@click.option('--verification-key-file', 'verification_key_file', required=True, type=str, help='Output filepath of the verification key.')
@click.option('--signing-key-file', 'signing_key_file', required=True, type=str, help='Output filepath of the signing key.')
def key_gen(verification_key_file, signing_key_file):
    """Create a stake address key pair"""
    subprocess.run(["cardano-cli",
                    "stake-address",
                    "key-gen",
                    "--verification-key-file", verification_key_file,
                    "--signing-key-file", signing_key_file
                    ])


__version__ = "0.1.5"
__license__ = "Apache-2.0 License"
__title__ = "cardanopy"

import click

from .run import run
from .create import create
from .cli import cli
from .docker import docker
from .k8s import k8s
from .config import config

@click.group()
@click.version_option(version=__version__)
@click.pass_context
def main(ctx):
    pass

# export
main.add_command(run)
main.add_command(create)
main.add_command(cli)
main.add_command(docker)
main.add_command(k8s)
main.add_command(config)



import click
import subprocess
from pathlib import Path
from cardanopy.cardanopy_config import CardanoPyConfig


@click.command()
@click.option('-r', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('--config-filename', 'config_filename', default='cardanopy.yaml', type=str, help="defaults to 'cardanopy.yaml'")
@click.argument('target_dir', type=str)
@click.pass_context
def exec(ctx, dry_run, target_dir, config_filename):
    """Docker Exec helper command"""

    if dry_run:
        print("#### DRY RUN - no mutable changes will be made. ####")

    target_dir = Path(target_dir)

    if not target_dir.is_dir():
        ctx.fail(f"Target directory '{target_dir}' is not a directory. e.g., the directory that contains 'cardanopy.yaml'")
        return 1

    if not target_dir.exists():
        ctx.fail(f"Target directory '{target_dir}' does not exist.")
        return 1

    target_config = target_dir.joinpath(config_filename)

    if not target_config.is_file():
        ctx.fail(
            f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")
        return 1

    if not target_config.exists():
        ctx.fail(f"Target file '{target_config}' does not exist.")
        return 1

    config = CardanoPyConfig()
    if not config.load(target_config):
        ctx.fail(f"Failed to load '{target_config}'")
        return 1

    try:
        result = subprocess.run(["docker",
                        "ps",
                        "-q",
                        "-f", f"name={config.name}"],
                        stdout=subprocess.PIPE).stdout.decode('utf-8')
        container_running = len(result) > 0
    except Exception as ex:
        ctx.fail(f"Unknown exception: {type(ex).__name__} {ex.args}")
        return 1

    if container_running:
        docker_run_cmd = ["docker",
                          "exec",
                          "-it",
                          config.name,
                          "bin/bash"]
        if dry_run:
            print(" ".join(docker_run_cmd))
        else:
            try:
                subprocess.run(docker_run_cmd)
            except Exception as ex:
                ctx.fail(f"Unknown exception: {type(ex).__name__} {ex.args}")
                return 1
    else:
        ctx.fail(f"Docker container named '{config.name}' is NOT currently running. Please 'run' the container first.")
        return 1



import click
import subprocess
from pathlib import Path
from .cardanopy_config import CardanoPyConfig
import json
import os
import shutil

@click.command()
@click.option('-r', '--dry-run', 'dry_run', is_flag=True, help="print the mutable commands")
@click.option('--config-filename', 'config_filename', default='cardanopy.yaml', type=str, help="defaults to 'cardanopy.yaml'")
@click.argument('target_dir', type=str)
@click.pass_context
def run(ctx, dry_run, target_dir, config_filename):
    """Run command"""

    if dry_run:
        print("#### DRY RUN - no mutable changes will be made. ####")

    target_dir = Path(target_dir)

    if not target_dir.is_dir():
        ctx.fail(f"Target directory '{target_dir}' is not a directory. e.g., the directory that contains 'cardanopy.yaml'")
        return 1

    if not target_dir.exists():
        ctx.fail(f"Target directory '{target_dir}' does not exist.")
        return 1

    target_config = target_dir.joinpath(config_filename)

    if not target_config.is_file():
        ctx.fail(
            f"Target config '{target_config}' is not a file. e.g., 'cardanopy.yaml'")
        return 1

    if not target_config.exists():
        ctx.fail(f"Target file '{target_config}' does not exist.")
        return 1

    config = CardanoPyConfig()
    if not config.load(target_config):
        ctx.fail(f"Failed to load '{target_config}'")
        return 1

    if not generate(dry_run, target_dir, config):
        ctx.fail(f"Failed to generate '{target_dir}'")
        return 1

    cardano_node_cmd = ["cardano-node",
                        "run",
                        "--config", config.config,
                        "--topology", config.topologyPath,
                        "--database-path", config.databasePath,
                        "--host-addr", config.hostAddr,
                        "--port", f"{config.port}",
                        "--socket-path", config.socketPath]
    if dry_run:
        print(" ".join(cardano_node_cmd))
    else:
        try:
            subprocess.run(cardano_node_cmd, cwd=target_dir)
        except Exception as ex:
            ctx.fail(f"Unknown exception: {ex} {type(ex).__name__} {ex.args}")
            return 1


def generate(dry_run: bool, target_dir: Path, config: CardanoPyConfig):
    if dry_run:
        print(f"generate 'topology.json' file config.topologyPath to `{config.topologyPath}'")
    else:
        with open(target_dir.joinpath(config.topologyPath), "w") as file:
            print(json.dumps(config.topology, sort_keys=True, indent=4), file=file)

        for dir_name, subdirList, fileList in os.walk(target_dir):
            dir_path = Path(dir_name)
            for file_name in fileList:
                file_path = dir_path.joinpath(file_name)
                if ".template" in file_name:
                    try:
                        output_template_yaml = dir_path.joinpath(file_name.replace(".template", ""))
                        if dry_run:
                            print(f"copy file from '{file_path}' to '{output_template_yaml}'")
                        else:
                            shutil.copyfile(file_path, output_template_yaml)
                    except Exception as ex:
                        print(f"Unknown exception: {ex} {type(ex).__name__} {ex.args}")
                        return False

    return True
